/* eslint-disable */

// @ts-nocheck

// noinspection JSUnusedGlobalSymbols

// This file was automatically generated by TanStack Router.
// You should NOT make any changes in this file as it will be overwritten.
// Additionally, you should also exclude this file from your linter and/or formatter to prevent it from being checked or modified.

import { Route as rootRouteImport } from './routes/__root'
import { Route as SettingsRouteImport } from './routes/settings'
import { Route as LogsRouteImport } from './routes/logs'
import { Route as IndexRouteImport } from './routes/index'
import { Route as QualityIndexRouteImport } from './routes/quality/index'
import { Route as HubIndexRouteImport } from './routes/hub/index'
import { Route as GraphIndexRouteImport } from './routes/graph/index'
import { Route as DataIndexRouteImport } from './routes/data/index'
import { Route as BranchesIndexRouteImport } from './routes/branches/index'
import { Route as AssetsIndexRouteImport } from './routes/assets/index'
import { Route as HubPluginsRouteImport } from './routes/hub/plugins'
import { Route as ExtensionsExtensionNameRouteImport } from './routes/extensions/$extensionName'
import { Route as DataBranchNameRouteImport } from './routes/data/$branchName'
import { Route as BranchesBranchNameRouteImport } from './routes/branches/$branchName'
import { Route as AssetsAssetIdRouteImport } from './routes/assets/$assetId'
import { Route as DataBranchNameIndexRouteImport } from './routes/data/$branchName/index'
import { Route as DataSchemaTableRouteImport } from './routes/data/$schema.$table'
import { Route as DataBranchNameSchemaTableRouteImport } from './routes/data/$branchName/$schema.$table'
import { Route as DataBranchNameSchemaTableRowIdRouteImport } from './routes/data/$branchName/$schema/$table/$rowId'

const SettingsRoute = SettingsRouteImport.update({
  id: '/settings',
  path: '/settings',
  getParentRoute: () => rootRouteImport,
} as any)
const LogsRoute = LogsRouteImport.update({
  id: '/logs',
  path: '/logs',
  getParentRoute: () => rootRouteImport,
} as any)
const IndexRoute = IndexRouteImport.update({
  id: '/',
  path: '/',
  getParentRoute: () => rootRouteImport,
} as any)
const QualityIndexRoute = QualityIndexRouteImport.update({
  id: '/quality/',
  path: '/quality/',
  getParentRoute: () => rootRouteImport,
} as any)
const HubIndexRoute = HubIndexRouteImport.update({
  id: '/hub/',
  path: '/hub/',
  getParentRoute: () => rootRouteImport,
} as any)
const GraphIndexRoute = GraphIndexRouteImport.update({
  id: '/graph/',
  path: '/graph/',
  getParentRoute: () => rootRouteImport,
} as any)
const DataIndexRoute = DataIndexRouteImport.update({
  id: '/data/',
  path: '/data/',
  getParentRoute: () => rootRouteImport,
} as any)
const BranchesIndexRoute = BranchesIndexRouteImport.update({
  id: '/branches/',
  path: '/branches/',
  getParentRoute: () => rootRouteImport,
} as any)
const AssetsIndexRoute = AssetsIndexRouteImport.update({
  id: '/assets/',
  path: '/assets/',
  getParentRoute: () => rootRouteImport,
} as any)
const HubPluginsRoute = HubPluginsRouteImport.update({
  id: '/hub/plugins',
  path: '/hub/plugins',
  getParentRoute: () => rootRouteImport,
} as any)
const ExtensionsExtensionNameRoute = ExtensionsExtensionNameRouteImport.update({
  id: '/extensions/$extensionName',
  path: '/extensions/$extensionName',
  getParentRoute: () => rootRouteImport,
} as any)
const DataBranchNameRoute = DataBranchNameRouteImport.update({
  id: '/data/$branchName',
  path: '/data/$branchName',
  getParentRoute: () => rootRouteImport,
} as any)
const BranchesBranchNameRoute = BranchesBranchNameRouteImport.update({
  id: '/branches/$branchName',
  path: '/branches/$branchName',
  getParentRoute: () => rootRouteImport,
} as any)
const AssetsAssetIdRoute = AssetsAssetIdRouteImport.update({
  id: '/assets/$assetId',
  path: '/assets/$assetId',
  getParentRoute: () => rootRouteImport,
} as any)
const DataBranchNameIndexRoute = DataBranchNameIndexRouteImport.update({
  id: '/',
  path: '/',
  getParentRoute: () => DataBranchNameRoute,
} as any)
const DataSchemaTableRoute = DataSchemaTableRouteImport.update({
  id: '/data/$schema/$table',
  path: '/data/$schema/$table',
  getParentRoute: () => rootRouteImport,
} as any)
const DataBranchNameSchemaTableRoute =
  DataBranchNameSchemaTableRouteImport.update({
    id: '/$schema/$table',
    path: '/$schema/$table',
    getParentRoute: () => DataBranchNameRoute,
  } as any)
const DataBranchNameSchemaTableRowIdRoute =
  DataBranchNameSchemaTableRowIdRouteImport.update({
    id: '/$rowId',
    path: '/$rowId',
    getParentRoute: () => DataBranchNameSchemaTableRoute,
  } as any)

export interface FileRoutesByFullPath {
  '/': typeof IndexRoute
  '/logs': typeof LogsRoute
  '/settings': typeof SettingsRoute
  '/assets/$assetId': typeof AssetsAssetIdRoute
  '/branches/$branchName': typeof BranchesBranchNameRoute
  '/data/$branchName': typeof DataBranchNameRouteWithChildren
  '/extensions/$extensionName': typeof ExtensionsExtensionNameRoute
  '/hub/plugins': typeof HubPluginsRoute
  '/assets': typeof AssetsIndexRoute
  '/branches': typeof BranchesIndexRoute
  '/data': typeof DataIndexRoute
  '/graph': typeof GraphIndexRoute
  '/hub': typeof HubIndexRoute
  '/quality': typeof QualityIndexRoute
  '/data/$schema/$table': typeof DataSchemaTableRoute
  '/data/$branchName/': typeof DataBranchNameIndexRoute
  '/data/$branchName/$schema/$table': typeof DataBranchNameSchemaTableRouteWithChildren
  '/data/$branchName/$schema/$table/$rowId': typeof DataBranchNameSchemaTableRowIdRoute
}
export interface FileRoutesByTo {
  '/': typeof IndexRoute
  '/logs': typeof LogsRoute
  '/settings': typeof SettingsRoute
  '/assets/$assetId': typeof AssetsAssetIdRoute
  '/branches/$branchName': typeof BranchesBranchNameRoute
  '/extensions/$extensionName': typeof ExtensionsExtensionNameRoute
  '/hub/plugins': typeof HubPluginsRoute
  '/assets': typeof AssetsIndexRoute
  '/branches': typeof BranchesIndexRoute
  '/data': typeof DataIndexRoute
  '/graph': typeof GraphIndexRoute
  '/hub': typeof HubIndexRoute
  '/quality': typeof QualityIndexRoute
  '/data/$schema/$table': typeof DataSchemaTableRoute
  '/data/$branchName': typeof DataBranchNameIndexRoute
  '/data/$branchName/$schema/$table': typeof DataBranchNameSchemaTableRouteWithChildren
  '/data/$branchName/$schema/$table/$rowId': typeof DataBranchNameSchemaTableRowIdRoute
}
export interface FileRoutesById {
  __root__: typeof rootRouteImport
  '/': typeof IndexRoute
  '/logs': typeof LogsRoute
  '/settings': typeof SettingsRoute
  '/assets/$assetId': typeof AssetsAssetIdRoute
  '/branches/$branchName': typeof BranchesBranchNameRoute
  '/data/$branchName': typeof DataBranchNameRouteWithChildren
  '/extensions/$extensionName': typeof ExtensionsExtensionNameRoute
  '/hub/plugins': typeof HubPluginsRoute
  '/assets/': typeof AssetsIndexRoute
  '/branches/': typeof BranchesIndexRoute
  '/data/': typeof DataIndexRoute
  '/graph/': typeof GraphIndexRoute
  '/hub/': typeof HubIndexRoute
  '/quality/': typeof QualityIndexRoute
  '/data/$schema/$table': typeof DataSchemaTableRoute
  '/data/$branchName/': typeof DataBranchNameIndexRoute
  '/data/$branchName/$schema/$table': typeof DataBranchNameSchemaTableRouteWithChildren
  '/data/$branchName/$schema/$table/$rowId': typeof DataBranchNameSchemaTableRowIdRoute
}
export interface FileRouteTypes {
  fileRoutesByFullPath: FileRoutesByFullPath
  fullPaths:
    | '/'
    | '/logs'
    | '/settings'
    | '/assets/$assetId'
    | '/branches/$branchName'
    | '/data/$branchName'
    | '/extensions/$extensionName'
    | '/hub/plugins'
    | '/assets'
    | '/branches'
    | '/data'
    | '/graph'
    | '/hub'
    | '/quality'
    | '/data/$schema/$table'
    | '/data/$branchName/'
    | '/data/$branchName/$schema/$table'
    | '/data/$branchName/$schema/$table/$rowId'
  fileRoutesByTo: FileRoutesByTo
  to:
    | '/'
    | '/logs'
    | '/settings'
    | '/assets/$assetId'
    | '/branches/$branchName'
    | '/extensions/$extensionName'
    | '/hub/plugins'
    | '/assets'
    | '/branches'
    | '/data'
    | '/graph'
    | '/hub'
    | '/quality'
    | '/data/$schema/$table'
    | '/data/$branchName'
    | '/data/$branchName/$schema/$table'
    | '/data/$branchName/$schema/$table/$rowId'
  id:
    | '__root__'
    | '/'
    | '/logs'
    | '/settings'
    | '/assets/$assetId'
    | '/branches/$branchName'
    | '/data/$branchName'
    | '/extensions/$extensionName'
    | '/hub/plugins'
    | '/assets/'
    | '/branches/'
    | '/data/'
    | '/graph/'
    | '/hub/'
    | '/quality/'
    | '/data/$schema/$table'
    | '/data/$branchName/'
    | '/data/$branchName/$schema/$table'
    | '/data/$branchName/$schema/$table/$rowId'
  fileRoutesById: FileRoutesById
}
export interface RootRouteChildren {
  IndexRoute: typeof IndexRoute
  LogsRoute: typeof LogsRoute
  SettingsRoute: typeof SettingsRoute
  AssetsAssetIdRoute: typeof AssetsAssetIdRoute
  BranchesBranchNameRoute: typeof BranchesBranchNameRoute
  DataBranchNameRoute: typeof DataBranchNameRouteWithChildren
  ExtensionsExtensionNameRoute: typeof ExtensionsExtensionNameRoute
  HubPluginsRoute: typeof HubPluginsRoute
  AssetsIndexRoute: typeof AssetsIndexRoute
  BranchesIndexRoute: typeof BranchesIndexRoute
  DataIndexRoute: typeof DataIndexRoute
  GraphIndexRoute: typeof GraphIndexRoute
  HubIndexRoute: typeof HubIndexRoute
  QualityIndexRoute: typeof QualityIndexRoute
  DataSchemaTableRoute: typeof DataSchemaTableRoute
}

declare module '@tanstack/react-router' {
  interface FileRoutesByPath {
    '/settings': {
      id: '/settings'
      path: '/settings'
      fullPath: '/settings'
      preLoaderRoute: typeof SettingsRouteImport
      parentRoute: typeof rootRouteImport
    }
    '/logs': {
      id: '/logs'
      path: '/logs'
      fullPath: '/logs'
      preLoaderRoute: typeof LogsRouteImport
      parentRoute: typeof rootRouteImport
    }
    '/': {
      id: '/'
      path: '/'
      fullPath: '/'
      preLoaderRoute: typeof IndexRouteImport
      parentRoute: typeof rootRouteImport
    }
    '/quality/': {
      id: '/quality/'
      path: '/quality'
      fullPath: '/quality'
      preLoaderRoute: typeof QualityIndexRouteImport
      parentRoute: typeof rootRouteImport
    }
    '/hub/': {
      id: '/hub/'
      path: '/hub'
      fullPath: '/hub'
      preLoaderRoute: typeof HubIndexRouteImport
      parentRoute: typeof rootRouteImport
    }
    '/graph/': {
      id: '/graph/'
      path: '/graph'
      fullPath: '/graph'
      preLoaderRoute: typeof GraphIndexRouteImport
      parentRoute: typeof rootRouteImport
    }
    '/data/': {
      id: '/data/'
      path: '/data'
      fullPath: '/data'
      preLoaderRoute: typeof DataIndexRouteImport
      parentRoute: typeof rootRouteImport
    }
    '/branches/': {
      id: '/branches/'
      path: '/branches'
      fullPath: '/branches'
      preLoaderRoute: typeof BranchesIndexRouteImport
      parentRoute: typeof rootRouteImport
    }
    '/assets/': {
      id: '/assets/'
      path: '/assets'
      fullPath: '/assets'
      preLoaderRoute: typeof AssetsIndexRouteImport
      parentRoute: typeof rootRouteImport
    }
    '/hub/plugins': {
      id: '/hub/plugins'
      path: '/hub/plugins'
      fullPath: '/hub/plugins'
      preLoaderRoute: typeof HubPluginsRouteImport
      parentRoute: typeof rootRouteImport
    }
    '/extensions/$extensionName': {
      id: '/extensions/$extensionName'
      path: '/extensions/$extensionName'
      fullPath: '/extensions/$extensionName'
      preLoaderRoute: typeof ExtensionsExtensionNameRouteImport
      parentRoute: typeof rootRouteImport
    }
    '/data/$branchName': {
      id: '/data/$branchName'
      path: '/data/$branchName'
      fullPath: '/data/$branchName'
      preLoaderRoute: typeof DataBranchNameRouteImport
      parentRoute: typeof rootRouteImport
    }
    '/branches/$branchName': {
      id: '/branches/$branchName'
      path: '/branches/$branchName'
      fullPath: '/branches/$branchName'
      preLoaderRoute: typeof BranchesBranchNameRouteImport
      parentRoute: typeof rootRouteImport
    }
    '/assets/$assetId': {
      id: '/assets/$assetId'
      path: '/assets/$assetId'
      fullPath: '/assets/$assetId'
      preLoaderRoute: typeof AssetsAssetIdRouteImport
      parentRoute: typeof rootRouteImport
    }
    '/data/$branchName/': {
      id: '/data/$branchName/'
      path: '/'
      fullPath: '/data/$branchName/'
      preLoaderRoute: typeof DataBranchNameIndexRouteImport
      parentRoute: typeof DataBranchNameRoute
    }
    '/data/$schema/$table': {
      id: '/data/$schema/$table'
      path: '/data/$schema/$table'
      fullPath: '/data/$schema/$table'
      preLoaderRoute: typeof DataSchemaTableRouteImport
      parentRoute: typeof rootRouteImport
    }
    '/data/$branchName/$schema/$table': {
      id: '/data/$branchName/$schema/$table'
      path: '/$schema/$table'
      fullPath: '/data/$branchName/$schema/$table'
      preLoaderRoute: typeof DataBranchNameSchemaTableRouteImport
      parentRoute: typeof DataBranchNameRoute
    }
    '/data/$branchName/$schema/$table/$rowId': {
      id: '/data/$branchName/$schema/$table/$rowId'
      path: '/$rowId'
      fullPath: '/data/$branchName/$schema/$table/$rowId'
      preLoaderRoute: typeof DataBranchNameSchemaTableRowIdRouteImport
      parentRoute: typeof DataBranchNameSchemaTableRoute
    }
  }
}

interface DataBranchNameSchemaTableRouteChildren {
  DataBranchNameSchemaTableRowIdRoute: typeof DataBranchNameSchemaTableRowIdRoute
}

const DataBranchNameSchemaTableRouteChildren: DataBranchNameSchemaTableRouteChildren =
  {
    DataBranchNameSchemaTableRowIdRoute: DataBranchNameSchemaTableRowIdRoute,
  }

const DataBranchNameSchemaTableRouteWithChildren =
  DataBranchNameSchemaTableRoute._addFileChildren(
    DataBranchNameSchemaTableRouteChildren,
  )

interface DataBranchNameRouteChildren {
  DataBranchNameIndexRoute: typeof DataBranchNameIndexRoute
  DataBranchNameSchemaTableRoute: typeof DataBranchNameSchemaTableRouteWithChildren
}

const DataBranchNameRouteChildren: DataBranchNameRouteChildren = {
  DataBranchNameIndexRoute: DataBranchNameIndexRoute,
  DataBranchNameSchemaTableRoute: DataBranchNameSchemaTableRouteWithChildren,
}

const DataBranchNameRouteWithChildren = DataBranchNameRoute._addFileChildren(
  DataBranchNameRouteChildren,
)

const rootRouteChildren: RootRouteChildren = {
  IndexRoute: IndexRoute,
  LogsRoute: LogsRoute,
  SettingsRoute: SettingsRoute,
  AssetsAssetIdRoute: AssetsAssetIdRoute,
  BranchesBranchNameRoute: BranchesBranchNameRoute,
  DataBranchNameRoute: DataBranchNameRouteWithChildren,
  ExtensionsExtensionNameRoute: ExtensionsExtensionNameRoute,
  HubPluginsRoute: HubPluginsRoute,
  AssetsIndexRoute: AssetsIndexRoute,
  BranchesIndexRoute: BranchesIndexRoute,
  DataIndexRoute: DataIndexRoute,
  GraphIndexRoute: GraphIndexRoute,
  HubIndexRoute: HubIndexRoute,
  QualityIndexRoute: QualityIndexRoute,
  DataSchemaTableRoute: DataSchemaTableRoute,
}
export const routeTree = rootRouteImport
  ._addFileChildren(rootRouteChildren)
  ._addFileTypes<FileRouteTypes>()

import type { getRouter } from './router.tsx'
import type { createStart } from '@tanstack/react-start'
declare module '@tanstack/react-start' {
  interface Register {
    ssr: true
    router: Awaited<ReturnType<typeof getRouter>>
  }
}
